/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright (c) 1999 Ming Lu, computer science, cornell university
 * All rights reserved.
 *
 * This package can be used for non-profit, non-commercial
 * purpose. For any other purpose, please contact the author at 
 * ml124@cornell.edu
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*******************************
 *
 *  MPEGVideo.cc
 *
 *  $Id: MPEGVideo.cc,v 3.0 1999/05/13 16:55:36 minglu Exp minglu $
 *  $log$
 *
 *******************************/

#include <assert.h>
#include "MPEGVideo.h"
#include <errno.h>
extern "C"
{
   #include "io-util.h"

};
#include "unix-util.cc"

MPEGVideo::MPEGVideo()
{
   
}

MPEGVideo::MPEGVideo(char *filename)
{
   strcpy(movieName,filename);
   inDecoding=0;
   
}

MPEGVideo::MPEGVideo(const MPEGVideo & that)
{
   copy(that);
}

MPEGVideo & MPEGVideo::operator=(const MPEGVideo &that)
{
   
   if( this==&that)   // note: this is ONLY good for single inheritance
      return *this;  
   
       // IMPORTANT
       // if this is a child class of some base class Base, then do this first :
       //  Base::operator=(that);

       
   cleanup(); // in the case of inheritance, cleanup and copy can be VIRTUAL FUNCTIONS !!
   copy(that);
      
   return *this;
}

int MPEGVideo::operator==(const MPEGVideo &right)
{
   
}

int MPEGVideo::operator!=(const MPEGVideo &right)
{
   return !(operator==(right));
}

MPEGVideo::~MPEGVideo()
{
   cleanup();
}

// delete the memory used by this class
void MPEGVideo::cleanup()
{
   
}

void MPEGVideo::copy(const MPEGVideo & that)
{
   
}

void MPEGVideo::debug(void * p)
{
#ifdef DEBUG 
   assert(p);
   cerr<<__LINE__;
#endif
}

void MPEGVideo::getFrame(int seqno,unsigned char *frame, int &rows, int &cols, int &colors)
{
   if(ifDecoded())
      {
      }
   else
      {
             // mpeg2decode -f -x ref%d -b bitstream.mpg -o0 rec%d
             //decode it
         if(inDecoding==0)
            {
               
               char* args[8];
               for(int i=0;i<8;i++)
                  args[i]=new (char)[20];
               
               strcpy(args[0],"mpeg2decode");
               strcpy(args[1],"-b");
               strcpy(args[2],movieName);
               
               strcpy(args[3],"-f");
               strcpy(args[4],"-r");
               strcpy(args[5],"-o3");
               strcpy(args[6],movieName);
               strcat(args[6],"%d");
               strcpy(args[7],"-q");
               
//               printf("start to decode...\n");

               execute(args);
//               printf("end of decode ...\n");
               
               
               inDecoding=1;
            }
         char ppmname[24];

         FILE *fp;
         do
            {
               sprintf(ppmname,"%s%d.ppm",movieName,seqno+1);
               fp=fopen(ppmname,"r");
               usleep(500000);
            }while(fp==NULL);
         fclose(fp);
         sprintf(ppmname,"%s%d.ppm",movieName,seqno);
         read_ppm_BYTE_withpreallcatedMemory(ppmname,frame,&rows,&cols);
         
//         printf("*********** rows=%d cols=%d \n",rows,cols);
         
         colors=3;
      }
   
}
int MPEGVideo::ifDecoded()
{
   return 0;
}
