/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright (c) 1999 Ming Lu, computer science, cornell university
 * All rights reserved.
 *
 * This package can be used for non-profit, non-commercial
 * purpose. For any other purpose, please contact the author at 
 * ml124@cornell.edu
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*******************************
 *
 *  PyramidEncoder.h
 *
 *  $Id: PyramidEncoder.h,v 1.3 1999/05/11 00:53:08 minglu Exp $
 *  $log$
 *
 *******************************/

#ifndef PyramidEncoder_H
#define PyramidEncoder_H

#include "LayerEncoder.h"
#include "ChannelBW.h"

struct _PyramidInfo
{
   int width;
   int height;
   int colors;
   int length; //total length
   int noOfBlocks;
   int len[3];//length of each block
   unsigned char *data;//data for each block
};


class PyramidEncoder:public LayerEncoder
{
private:
   void copy(const PyramidEncoder & src);
   void cleanup();

public:
   
   PyramidEncoder();
   PyramidEncoder(int _noOfChannels);
   
   PyramidEncoder(ChannelBW &bw);
   PyramidEncoder(const PyramidEncoder & that);
   virtual ~PyramidEncoder();

   PyramidEncoder & operator=(const PyramidEncoder &that);
   int operator==(const PyramidEncoder &right);
   int operator!=(const PyramidEncoder &right);

   void debug(void * p);

   void PyramidEncoder::encode(unsigned char *frame,
                               int seqno,
                               int rows,
                               int cols,
                               int colors,
                               unsigned char **out,
                               int* len);
   
};

#endif /* PyramidEncoder_H */

