/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright (c) 1999 Ming Lu, computer science, cornell university
 * All rights reserved.
 *
 * This package can be used for non-profit, non-commercial
 * purpose. For any other purpose, please contact the author at 
 * ml124@cornell.edu
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*******************************
 *
 *  SRMChannelSever.cc
 *
 *  $Id: SRMChannelSever.cc,v 2.3 1999/05/09 18:51:47 minglu Exp $
 *  $log$
 *
 *******************************/


#include <strings.h>
#include "SRMChannelSever.h"
#include <stdio.h>


SRMChannelSever::SRMChannelSever()
{
}

SRMChannelSever::SRMChannelSever(unsigned int _addr,
                     unsigned int _port,
                     unsigned int _ttl
                   )
{
   init(_addr,_port,_ttl);
}

void SRMChannelSever::init(unsigned int _addr,
                           unsigned int _port,
                           unsigned int _ttl
                           )   
{
   addr=_addr;
   port=_port;
   ttl=_ttl;
       // allocate channels
   session = srm_create_session(addr, port, port, ttl);
   srm_set_drop_probability(session, 0.2);
   source= srm_create_source(session, NULL);
   cid=SRMv2_ROOT_CID;
   firstTime=1;
   cidStepSize=0;
   base=cid;
   
}

void SRMChannelSever::setCallbacks(srm_callbacks &callbacks)
{
   srm_set_callbacks(session, &callbacks);  
}


void SRMChannelSever::cleanup()
{
}

SRMChannelSever::~SRMChannelSever()
{
   cleanup();
}

void SRMChannelSever::sendData(srm_source_t _source,int channelID, int seqNo, unsigned char *data, int len)
{
   int cid=encodeCid(_source,seqNo,channelID);
   srm_send(_source, cid, data,len,NULL); 
}

void SRMChannelSever::sendData(int channelID, int seqNo, unsigned char *data, int len)
{

   int ccid=encodeCid(seqNo,channelID);
   srm_send(source, ccid, data,len,NULL); 
}

void  SRMChannelSever::decipher(srm_source_t _source,int cid, int &channelID, int & seqNo)
{
   unsigned char dataName[24];
   
   char  *p;
   const unsigned char *cname;
   int l;
   
   strcpy(dataName, "");
   p = dataName + strlen(dataName);
   while (cid != SRMv2_ROOT_CID) {
      srm_get_container_name(_source, cid, &cname, NULL);
      
      char **cnames;
      int cc;
      Tcl_SplitPath(cname, &cc, &cnames);
                char *comp = cnames[cc-1];

                if (*p) {
                        l = strlen(comp);
                        memmove(p+l+1, p, strlen(p)+1);
                        memmove(p, comp, l);
                        *(p+l) = '-';
                } else {
                        strcpy(p, comp);
                }
                Tcl_Free((char *)cnames);
                cid = srm_get_parent_cid(_source, cid);
        }

//   srm_get_container_name(source[0], cid, &cname, NULL);
   printf("findChannelNo:: %s\n",dataName);
   
   for(int i=strlen(dataName);i>=0;i--)
      {
         if(dataName[i]=='-') {
            
         sscanf(dataName+i+1,"%d",&seqNo);
         dataName[i]='\0';
         }
      }  
   sscanf(dataName,"%d",&channelID);
//   printf("channelID=%d seqNo=%d \n",channelID,seqNo);
   
}
int SRMChannelSever::encodeCid(int seqNo,int channelID)
{
       // int cid=SRMv2_ROOT_CID;
   char tmpString[50];
   if(cidStepSize==48)
      {
         
      firstTime=1;
      cidStepSize=0;
   }
   
   if(firstTime)
      {
         sprintf(tmpString,"%d",channelID);
         cid=srm_calloc(source, base , (const unsigned char *)tmpString, strlen(tmpString)+1);
         base=cid;
         firstTime=0;
      }
   sprintf(tmpString,"%d",seqNo);   
   cid = srm_calloc(source, base , (const unsigned char *)tmpString, strlen(tmpString)+1);
   cidStepSize++;
   
// cid = srm_calloc(source, cid , (const unsigned char *)tmpString, strlen(tmpString)+1);
// printf("cid=%d \n",cid);
 
 return cid;
   
//   return encodeCid(source,seqNo,channelID);
}

int SRMChannelSever::encodeCid(srm_source_t _source,int seqNo,int channelID)
{
 int cid=SRMv2_ROOT_CID;

 char tmpString[20];
 sprintf(tmpString,"%d",channelID);
 cid = srm_calloc(_source, cid , (const unsigned char *)tmpString, strlen(tmpString)+1);
 sprintf(tmpString,"%d",seqNo);
 cid = srm_calloc(_source, cid , (const unsigned char *)tmpString, strlen(tmpString)+1);
 return cid;
}


void SRMChannelSever::decipher(int cid,int &channelID, int &seqNo)
{
   decipher(source,cid,channelID,seqNo);
}
