/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright (c) 1999 Ming Lu, computer science, cornell university
 * All rights reserved.
 *
 * This package can be used for non-profit, non-commercial
 * purpose. For any other purpose, please contact the author at 
 * ml124@cornell.edu
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*******************************
 *
 *  SRMClient.cc
 *
 *  $Id: SRMClient.cc,v 1.4 1999/05/02 00:14:10 minglu Exp $
 *  $log$
 *
 *******************************/

#include <stdio.h>
#include "SRMClient.h"
unsigned int SRMClient::addr;
unsigned int SRMClient::port;
unsigned int SRMClient::ttl;
int SRMClient::noOfChannels;
char **SRMClient::dataReceived;
int *SRMClient::lenReceived;
int *SRMClient::seqnoReceived;
float *SRMClient::bandwidth;
srm_session_t *SRMClient::session;
srm_source_t *SRMClient::source;

SRMClient::SRMClient()
{
}

SRMClient::SRMClient(unsigned int _addr,
                     unsigned int _port,
                     unsigned int _ttl,
                     int _noOfChannels)
{
   init(_addr,_port,_ttl,_noOfChannels);
}

void SRMClient::init(unsigned int _addr,
                     unsigned int _port,
                     unsigned int _ttl,
                     int _noOfChannels)   
{
   addr=_addr;
   port=_port;
   ttl=_ttl;
   noOfChannels=_noOfChannels;
   

   dataReceived=new (char *)[noOfChannels];
   lenReceived=new (int)[noOfChannels];
   seqnoReceived=new (int)[noOfChannels];
   bandwidth=new (float)[noOfChannels];
   
   session=new srm_session_t[noOfChannels];
   source=new  srm_source_t[noOfChannels];


   for(int i=0;i<noOfChannels;i++)
      {
         session[i] = srm_create_session(addr, port, port, ttl);
         source[i] = srm_create_source(session[i], NULL);
      }
   
}
void SRMClient::setCallbacks(srm_callbacks &callbacks)
{
  
   for(int i=0;i<noOfChannels;i++)
      {
         srm_set_callbacks(session[i], &callbacks);
      } 
}

void SRMClient::setProbability(int channelNo,float probabability)
{
    srm_set_drop_probability(session[channelNo], 0.2); 
 }


//The SRMSever is supposed to encode the channelNo in the cid, then decode cid, knows where to put the data

int SRMClient::findChannelNo(srm_source_t _source,int cid,char * dataName)
{
   

//   printf("---------\n");

   char  *p;
   const unsigned char *cname;
   int l;
   strcpy(dataName, "");
   p = dataName + strlen(dataName);
   while (cid != SRMv2_ROOT_CID) {
      srm_get_container_name(_source, cid, &cname, NULL);
      
      char **cnames;
      int cc;
      Tcl_SplitPath(cname, &cc, &cnames);
                char *comp = cnames[cc-1];

                if (*p) {
                        l = strlen(comp);
                        memmove(p+l+1, p, strlen(p)+1);
                        memmove(p, comp, l);
                        *(p+l) = '-';
                } else {
                        strcpy(p, comp);
                }
                Tcl_Free((char *)cnames);
                cid = srm_get_parent_cid(_source, cid);
        }




   
//   srm_get_container_name(source[0], cid, &cname, NULL);
//   printf("findChannelNo:: %s\n",dataName);
   
   for(int i=strlen(dataName);i>=0;i--)
      {
         if(dataName[i]=='-') {
            
         int x;
         sscanf(dataName+i+1,"%d",&x);

         
         dataName[i]='\0';
         return x;   
         }
         
      }
}
void SRMClient::cleanup()
{
   for(int i=0;i<noOfChannels;i++)
      {
         delete[] dataReceived[i];
      }
   delete[] session;
   delete[] source;
   delete[] lenReceived;
   delete[] seqnoReceived;
   delete[] bandwidth;
}


SRMClient::~SRMClient()
{
   cleanup();
}

void SRMClient::setDataReceived(int channelNo,const unsigned char *_dataReceived, int _len,int _seqno)
{
   memcpy(dataReceived[channelNo],_dataReceived,_len);
   lenReceived[channelNo]=_len;
   seqnoReceived[channelNo]=_seqno;
}


char * SRMClient::getDataReceived(int channelNo)
{
   return dataReceived[channelNo];
}

int SRMClient::getLenReceived(int channelNo)
{
   return lenReceived[channelNo];
}

int SRMClient::getSeqnoReceived(int channelNo)
{
   return seqnoReceived[channelNo];
}


