/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright (c) 1999 Ming Lu, computer science, cornell university
 * All rights reserved.
 *
 * This package can be used for non-profit, non-commercial
 * purpose. For any other purpose, please contact the author at 
 * ml124@cornell.edu
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*******************************
 *
 *  VideoSRMChannelClient.h
 *
 *  $Id: VideoSRMChannelClient.h,v 2.1 1999/05/05 22:59:18 minglu Exp $
 *  $log$
 *
 *******************************/

#ifndef VideoSRMChannelClient_H
#define VideoSRMChannelClient_H
class VideoSRMClient;

#include "SRMChannelClient.h"
#include "VideoSRMClient.h"

class VideoSRMChannelClient:public SRMChannelClient
{
private:
   void copy(const VideoSRMChannelClient & src);
   void cleanup();
       // this is used to save the higher channels for this later when low channel data is received

   static VideoSRMClient* videoClient;

   
public:
   
   VideoSRMChannelClient();
   VideoSRMChannelClient(unsigned int _addr,
                         unsigned int _port,
                         unsigned int _ttl,
                         int _channelID,
                         VideoSRMClient * _videoClient);
   void init(unsigned int _addr,
             unsigned int _port,
             unsigned int _ttl,
             int _channelID);

   
   VideoSRMChannelClient(const VideoSRMChannelClient & that);
   virtual ~VideoSRMChannelClient();

   VideoSRMChannelClient & operator=(const VideoSRMChannelClient &that);
   int operator==(const VideoSRMChannelClient &right);
   int operator!=(const VideoSRMChannelClient &right);


   
   void debug(void * p);
   
   static void srm_recv(srm_source_t source,
                        unsigned int cid,
                        unsigned int seqno ,
                        const unsigned char *data,
                        int len,
                        const srm_adu_info * info);
   
   static int srm_should_recover(srm_source_t source,
                                 unsigned int cid,
                                 unsigned int ss,
                                 unsigned int es);
   
   static void srm_read_adu(srm_source_t source,
                            unsigned int cid,
                            unsigned int seqno ,
                            unsigned char **data_ptr,
                            unsigned int *len_ptr,
                            srm_free_proc *free_proc_ptr,
                            srm_adu_info * info);


   static void srm_source_update(srm_source_t source ,
                                 const unsigned char *info,
                                 int len);

};

#endif /* VideoSRMChannelClient_H */

