/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright (c) 1999 Ming Lu, computer science, cornell university
 * All rights reserved.
 *
 * This package can be used for non-profit, non-commercial
 * purpose. For any other purpose, please contact the author at 
 * ml124@cornell.edu
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*******************************
 *
 *  VideoSRMChannelSever.cc
 *
 *  $Id: VideoSRMChannelSever.cc,v 2.1 1999/05/05 22:59:01 minglu Exp $
 *  $log$
 *
 *******************************/

#include <iostream.h>
#include <assert.h>
#include "VideoSRMChannelSever.h"
#include "TestVideo.h"



VideoSRMSever * VideoSRMChannelSever::videoSever;
int VideoSRMChannelSever::ok;

VideoSRMChannelSever::VideoSRMChannelSever()
{
   
}

VideoSRMChannelSever::VideoSRMChannelSever(
   unsigned int _addr,
   unsigned int _port,
   unsigned int _ttl,
   int _channelID,
   VideoSRMSever *_videoSever)
   :SRMChannelSever(_addr,_port,_ttl)
{
    init(_addr,_port,_ttl,_channelID);
    videoSever=_videoSever;
}
void VideoSRMChannelSever::init(
   unsigned int _addr,
   unsigned int _port,
   unsigned int _ttl,
   int _channelID)
{
   srm_callbacks callbacks= {
      srm_recv,
      srm_should_recover,
      srm_read_adu,
      srm_source_update
   };
   setCallbacks(callbacks);
   channelID=_channelID;
   ok=1;
   
}


   
VideoSRMChannelSever::VideoSRMChannelSever(const VideoSRMChannelSever & that)
{
   copy(that);
}

VideoSRMChannelSever & VideoSRMChannelSever::operator=(const VideoSRMChannelSever &that)
{
   
   if( this==&that)   // note: this is ONLY good for single inheritance
      return *this;  
   
       // IMPORTANT
       // if this is a child class of some base class Base, then do this first :
       //  Base::operator=(that);

       
   cleanup(); // in the case of inheritance, cleanup and copy can be VIRTUAL FUNCTIONS !!
   copy(that);
      
   return *this;
}

int VideoSRMChannelSever::operator==(const VideoSRMChannelSever &right)
{
   
}

int VideoSRMChannelSever::operator!=(const VideoSRMChannelSever &right)
{
   return !(operator==(right));
}

VideoSRMChannelSever::~VideoSRMChannelSever()
{
   cleanup();
}

// delete the memory used by this class
void VideoSRMChannelSever::cleanup()
{
   
}

void VideoSRMChannelSever::copy(const VideoSRMChannelSever & that)
{
   
}

void VideoSRMChannelSever::debug(void * p)
{
#ifdef DEBUG 
   assert(p);
   cerr<<__LINE__;
#endif
}
void VideoSRMChannelSever::srm_recv(srm_source_t source,
                                    unsigned int cid,
                                    unsigned int seqno ,
                                    const unsigned char *data,
                                    int len,
                                    const srm_adu_info * info)
{
//   printf("VideoSRMChannelSever::srm_recv(...) is called\n");
}

int VideoSRMChannelSever::srm_should_recover(srm_source_t source,
                                  unsigned int cid,
                                  unsigned int ss,
                                  unsigned int es)
{
   return 0;
}

// here the seqno is the frame number from 0 to ...
void VideoSRMChannelSever::srm_read_adu(
   srm_source_t source,
   unsigned int cid,
   unsigned int seqno ,
   unsigned char **data_ptr,
   unsigned int *len_ptr,
   srm_free_proc *free_proc_ptr,
   srm_adu_info * info)
{
//   printf("VideoSRMChannelSever::srm_read_adu(...) \n");
}


void VideoSRMChannelSever::srm_source_update(srm_source_t source ,
                                  const unsigned char *info,
                                  int len)
{
	if (info) {
		printf("\t %s\n", info);
	}
   
}


void VideoSRMChannelSever::_sendData(int channelID, int seqNo, unsigned char *data, int len)
{
         sendData(channelID,seqNo,data,len);
}

