/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright (c) 1999 Ming Lu, computer science, cornell university
 * All rights reserved.
 *
 * This package can be used for non-profit, non-commercial
 * purpose. For any other purpose, please contact the author at 
 * ml124@cornell.edu
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*******************************
 *
 *  VideoSRMClient.cc
 *
 *  $Id: VideoSRMClient.cc,v 2.3 1999/05/09 18:51:47 minglu Exp $
 *  $log$
 *
 *******************************/


#include <assert.h>
#include "VideoSRMClient.h"

VideoSRMClient::VideoSRMClient()
{
}

VideoSRMClient::VideoSRMClient(
   unsigned int *_addr,
   unsigned int *_port,
   unsigned int *_ttl,
   int _noOfChannels)
{
   noOfChannels=_noOfChannels;
   channels=new (VideoSRMChannelClient *)[_noOfChannels];
   for(int i=0;i<_noOfChannels;i++)
      {
         channels[i]=new VideoSRMChannelClient(_addr[i],_port[i],_ttl[i],i,this);
      }
   curFrame=new (unsigned char *)[noOfChannels];
   for(int i=0;i<noOfChannels;i++)
      {
         curFrame[i]=new (unsigned char)[MAX_WIDTH*MAX_HEIGHT*3];
      }
   
   lastframe=new int[noOfChannels];
   for(int i=0;i<noOfChannels;i++)
      lastframe[i]=-1;
   
   
}
void VideoSRMClient::init(
   unsigned int *_addr,
   unsigned int *_port,
   unsigned int *_ttl,
   int _noOfChannels)
{
}

void VideoSRMClient::setDecoder(LayerDecoder *_decoder)
{
   decoder=_decoder;
}
void VideoSRMClient::setVideoClient(VideoClient *_videoClient)
{
   videoClient=_videoClient;
}



VideoSRMClient::VideoSRMClient(const VideoSRMClient & that)
{
   copy(that);
}

VideoSRMClient & VideoSRMClient::operator=(const VideoSRMClient &that)
{
   
   if( this==&that)   // note: this is ONLY good for single inheritance
      return *this;  
   
       // IMPORTANT
       // if this is a child class of some base class Base, then do this first :
       //  Base::operator=(that);

       
   cleanup(); // in the case of inheritance, cleanup and copy can be VIRTUAL FUNCTIONS !!
   copy(that);
      
   return *this;
}

int VideoSRMClient::operator==(const VideoSRMClient &right)
{
   
}

int VideoSRMClient::operator!=(const VideoSRMClient &right)
{
   return !(operator==(right));
}

VideoSRMClient::~VideoSRMClient()
{
   cleanup();
}

// delete the memory used by this class
void VideoSRMClient::cleanup()
{
   
}

void VideoSRMClient::copy(const VideoSRMClient & that)
{
   
}



void VideoSRMClient::debug(void * p)
{
#ifdef DEBUG 
   assert(p);
   cerr<<__LINE__;
#endif
}



void VideoSRMClient::playVideo(int seqNo,int channelID, const unsigned char *data, int len)
{
   if(seqNo>(lastframe[channelID]))
      {
         
         int rows, cols, colors;
         unsigned char *image;
         memcpy(curFrame[channelID],data,len);
         if(channelID==(noOfChannels-1))
            {
               decoder->decode(curFrame,rows,cols,colors,&image);
               videoClient->play(image,rows,cols,colors);
            }
            
         (lastframe[channelID])++;
         
      }
   
}








