/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright (c) 1999 Ming Lu, computer science, cornell university
 * All rights reserved.
 *
 * This package can be used for non-profit, non-commercial
 * purpose. For any other purpose, please contact the author at 
 * ml124@cornell.edu
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*******************************
 *
 *  VideoSRMSever.cc
 *
 *  $Id: VideoSRMSever.cc,v 3.0 1999/05/13 16:55:36 minglu Exp minglu $
 *  $log$
 *
 *******************************/

#include <iostream.h>
#include <assert.h>
#include "VideoSRMSever.h"

#include "TestVideo.h"
#include <unistd.h>



VideoSRMSever::VideoSRMSever()
{
   
}

VideoSRMSever::VideoSRMSever(unsigned int *_addr,
                             unsigned int *_port,
                             unsigned int *_ttl,
                             int _noOfChannels)
{
    init(_addr,_port,_ttl,_noOfChannels);
}
void VideoSRMSever::init(unsigned int *_addr,
                         unsigned int *_port,
                         unsigned int *_ttl,
                         int _noOfChannels)
{
   
   noOfChannels=_noOfChannels;
   channels=new (VideoSRMChannelSever *)[_noOfChannels];
   for(int i=0;i<_noOfChannels;i++)
      {
         channels[i]=new VideoSRMChannelSever(_addr[i],_port[i],_ttl[i],i,this);
      }
   outLength=new int[noOfChannels];
   encodedOut=new (unsigned char *)[noOfChannels];
   frame=new (unsigned char)[MAX_WIDTH*MAX_HEIGHT*MAX_COLORS];
   encodedOutAllocated=0;
   
}
void VideoSRMSever::setEncoder(LayerEncoder &_encoder)
{
   encoder=&_encoder;
}
void VideoSRMSever::setVideo(Video* _video)
{
   video=_video;
}

   
VideoSRMSever::VideoSRMSever(const VideoSRMSever & that)
{
   copy(that);
}

VideoSRMSever & VideoSRMSever::operator=(const VideoSRMSever &that)
{
   
   if( this==&that)   // note: this is ONLY good for single inheritance
      return *this;  
   
       // IMPORTANT
       // if this is a child class of some base class Base, then do this first :
       //  Base::operator=(that);

       
   cleanup(); // in the case of inheritance, cleanup and copy can be VIRTUAL FUNCTIONS !!
   copy(that);
      
   return *this;
}

int VideoSRMSever::operator==(const VideoSRMSever &right)
{
   
}

int VideoSRMSever::operator!=(const VideoSRMSever &right)
{
   return !(operator==(right));
}

VideoSRMSever::~VideoSRMSever()
{
   cleanup();
}

// delete the memory used by this class
void VideoSRMSever::cleanup()
{
   
}

void VideoSRMSever::copy(const VideoSRMSever & that)
{
   
}

void VideoSRMSever::debug(void * p)
{
#ifdef DEBUG 
   assert(p);
   cerr<<__LINE__;
#endif
}


void VideoSRMSever::sendData(int frameNo)
{

   int rows,cols,colors;
   video->getFrame(frameNo,frame,rows, cols, colors);

   if(encodedOutAllocated==0)
      {
         
   for(int i=0;i<noOfChannels;i++)
      encodedOut[i]=new (unsigned char)[rows*cols*colors+51];
   encodedOutAllocated=1;
      }
   
   encoder->encode(frame,frameNo,rows, cols, colors,encodedOut, outLength);
 
   
   for(int i=0;i< noOfChannels;i++)
      {
         channels[i]->sendData(i,frameNo,encodedOut[i],outLength[i]);
      }
}

























































/*******************
void VideoSRMSever::srm_recv(srm_source_t source,
                        unsigned int cid,
                        unsigned int seqno ,
                        const unsigned char *data,
                        int len,
              const srm_adu_info * info)
{
   printf("VideoSRMSever::srm_recv(...) is called\n");
}

int VideoSRMSever::srm_should_recover(srm_source_t source,
                                  unsigned int cid,
                                  unsigned int ss,
                                  unsigned int es)
{
   return 1;
}

// here the seqno is the frame number from 0 to ...
void VideoSRMSever::srm_read_adu(
   srm_source_t source,
   unsigned int cid,
   unsigned int seqno ,
   unsigned char **data_ptr,
   unsigned int *len_ptr,
   srm_free_proc *free_proc_ptr,
   srm_adu_info * info)
{
       /////////////////////////////////
       //   1. get the data from VideoSource
       //   2. do the encoding
       //   3. encode the frame
       //   4. send the data crosponding to the cid
       /////////////////////////////////
   int channelNo,frameNo;
   decipher(cid, channelNo, frameNo);
   
   int rows,cols, colors;
   unsigned char* frame;
//   frame=video->getFrame(frameNo,rows, cols, colors);
       // use cid to decide which channel ?
   unsigned char **out;
   int *len;
   encoder->encode(frame,rows, cols, colors,out, len);
   
   *data_ptr=out[channelNo];
   *len_ptr=len[channelNo];
   info=NULL;
   
}


void VideoSRMSever::srm_source_update(srm_source_t source ,
                                  const unsigned char *info,
                                  int len)
{
	printf("In source update received \n");
    printf("len=%d\n",len);
    printf("Out source update received \n");
	if (info) {
		printf("\t %s\n", info);
	}
   
}

void VideoSRMSever::sendData(int frameNo,unsigned char *videoName)
{
   int rows,cols,colors;
   
   unsigned char* frame;
   
   video->getFrame(frameNo,&frame,rows, cols, colors);
   
   unsigned char **out;
   int *len;
   len=new int[noOfChannels];
   
   out=new (unsigned char *)[noOfChannels];
   for(int i=0;i<noOfChannels;i++)
      out[i]=new (unsigned char)[rows*cols*colors+1];
   printf("****************\n");
   
   encoder->encode(frame,rows, cols, colors,out, len);
   printf("****************\n");
   
   for(int channelNo=0;channelNo< noOfChannels;channelNo++)
//   for(int channelNo=channelNo-1;channelNo>=0;channelNo--)
      {
         int cid=encodeCid(channelNo,frameNo,videoName);
         if(out[channelNo]!=NULL)
            {
               srm_send(source[channelNo], cid, out[channelNo],len[channelNo],NULL);  
            }
         
         
      }
}
********/

/*****
int VideoSRMSever::encodeCid(int channelNo, int frameNo, unsigned char * videoName)
{
   
   int cid=SRMv2_ROOT_CID;
   cid = srm_calloc(source[channelNo], cid, (unsigned char *)videoName, strlen((const char *)videoName)+1);
   char tmpString[20];
   sprintf(tmpString,"%d",channelNo);
   cid = srm_calloc(source[channelNo], cid , (const unsigned char *)tmpString, strlen(tmpString)+1);
   sprintf(tmpString,"%d",frameNo);
   cid = srm_calloc(source[channelNo], cid , (const unsigned char *)tmpString, strlen(tmpString)+1);
   return cid;
   
}****/
/******
void VideoSRMSever::decipher(int cid, int &channelNo, int &frameNo) 
{
 char  *p;
   const unsigned char *cname;
   int l;
   
   strcpy(dataName, "");
   p = dataName + strlen(dataName);
   while (cid != SRMv2_ROOT_CID) {
      srm_get_container_name(_source, cid, &cname, NULL);
      
      char **cnames;
      int cc;
      Tcl_SplitPath(cname, &cc, &cnames);
                char *comp = cnames[cc-1];

                if (*p) {
                        l = strlen(comp);
                        memmove(p+l+1, p, strlen(p)+1);
                        memmove(p, comp, l);
                        *(p+l) = '-';
                } else {
                        strcpy(p, comp);
                }
                Tcl_Free((char *)cnames);
                cid = srm_get_parent_cid(_source, cid);
        }




   
//   srm_get_container_name(source[0], cid, &cname, NULL);
   printf("findChannelNo:: %s\n",dataName);
   
   for(int i=strlen(dataName);i>=0;i--)
      {
         if(dataName[i]=='-') {
            
         sscanf(dataName+i+1,"%d",&frameNo);
         dataName[i]='\0';
         }
      }  
   
 for(int i=strlen(dataName);i>=0;i--)
      {
         if(dataName[i]=='-') {
            
         sscanf(dataName+i+1,"%d",&channelNo);
         dataName[i]='\0';
         }
      }
 printf("channelNo=%d frameNo=%d \n",channelNo,frameNo);
 
}


****/
