/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright (c) 1999 Ming Lu, computer science, cornell university
 * All rights reserved.
 *
 * This package can be used for non-profit, non-commercial
 * purpose. For any other purpose, please contact the author at 
 * ml124@cornell.edu
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#include <stdio.h>

extern "C"
{
#include "io-util.h"
};
#include "PyramidEncoder.h"

void _encodeOnce(int _height, int _width,int _color,unsigned char *src,unsigned char *lowLow, unsigned char * lowHigh, unsigned char *highLow,unsigned char *highHigh, int *length)
{
   unsigned char *tSrc;
   int *tmp,*tmpHead, *tLow,*tHigh;
   int halfWidth=_width>>1;
   int size=_width*_height*_color;
   
   tmp=new int[size+1];
   tSrc=src;
   tmpHead=tmp;
       //do it horizontally
   for(int i=0;i<_height;i++,tmpHead+=_width*_color)
      {
         tLow=tmpHead;
         tHigh=tmpHead+ halfWidth*_color;
         
         for(int j=0;j<_width;j+=2,tSrc+=2*_color,tLow+=_color,tHigh+=_color)
            {
               for(int k=0;k<_color;k++)
                  {
                     
                     *(tLow+k) =((int)(*(tSrc+k)+*(tSrc+k+_color)))>>1;
                     *(tHigh+k)=((int)(*(tSrc+k+_color))-(int)(*(tSrc+k)))>>1;    
                  }
               
            }
         
      }
   
       // do it vertically
   tmpHead=tmp;
   unsigned char  *tmpLowLow,*tmpLowHigh,*tmpHighLow,*tmpHighHigh;
   

       ///////////////////////////////////
       //               //              //
       //               //              //
       //     Low Low   //   Low High   //
       //               //              //
       ///////////////////////////////////
       //               //              //
       //               //              //
       //     High Low  //   High High  //
       //               //              //
       ///////////////////////////////////   
    
   

   int *t;
   unsigned char *tmpLow,*tmpHigh;
   
       // do low low and low high
   tmpLowLow=lowLow;
   tmpHighLow=highLow;
   t=tmp;
   for(int i=0;i<halfWidth;i++,tmpLowLow+=_color,tmpHighLow+=_color,t=tmp+i*_color)
      {
         tmpLow=tmpLowLow;
         tmpHigh=tmpHighLow;
         for(int j=0;j<_height;j+=2,tmpLow+=halfWidth*_color,tmpHigh+=halfWidth*_color,t+=(_width<<1)*_color)
            {
               for(int k=0;k<_color;k++)
                  {
                     
                     *(tmpLow+k)=(unsigned char) ( (*(t+k)+*(t+k+_width*_color))>>1);
                     *(tmpHigh+k)=(unsigned char)((*(t+k+_width*_color)-(*(t+k))+256 )>>1 );
                  }
               
            }
         
      }
    // right half
   tmpLowHigh=lowHigh;
   tmpHighHigh=highHigh;
   t=tmp+halfWidth*_color;
   for(int i=halfWidth;i<_width;i++,tmpLowHigh+=_color,tmpHighHigh+=_color,t=tmp+i*_color)
      {
         tmpLow=tmpLowHigh;
         tmpHigh=tmpHighHigh;
         
         for(int j=0;j<_height;j+=2,tmpLow+=halfWidth*_color,tmpHigh+=halfWidth*_color,t+=(_width<<1)*_color)
            {

               for(int k=0;k<_color;k++)
                  {
                     
                     *(tmpLow+k)=(unsigned char ) ( ( *(t+k)+ *(t+k+_width*_color)  +256)>>1);
                     *(tmpHigh+k)=(unsigned char)( (*(t+k+_width*_color)-(*(t+k))+256)>>1);
                  }
               
            }
    }
   
   for(int i=0;i<4;i++)
      length[i]=size>>2;
   lowLow[size>>2]='\0';
   lowHigh[size>>2]='\0';
   highLow[size>>2]='\0';
   highHigh[size>>2]='\0';
   
}


void _encodeOnce(unsigned char *src, int height,int width,int colors,struct _PyramidInfo *low,struct _PyramidInfo *high)
{int length[4];
   unsigned char *tmp[4];
   for(int i=0;i<4;i++)
      tmp[i]=new (unsigned char)[width*height*colors+1];
   _encodeOnce(height,width,colors,src,tmp[0],tmp[1],tmp[2],tmp[3],length);

   low->width=(width>>1);
   low->height=(height>>1);
   low->colors=colors;
   low->noOfBlocks=1;
   low->length=length[0];
   low->len[0]=length[0];
   
//   low->data=tmp[0];
   low->data=new (unsigned char)[low->length+1];
   memcpy(low->data,tmp[0],low->length);
   
   high->width=(width>>1);
   high->height=(height>>1);
   high->colors=colors;
   high->noOfBlocks=3;
   high->length=length[1]+length[2]+length[3];
   for(int i=0;i<3;i++)
      high->len[i]=length[i+1];
   high->data=new (unsigned char)[high->length+1];
   unsigned char *t;
   t=high->data;
   for(int i=0;i<3;i++)
      {
         memcpy(t,tmp[i+1],length[i+1]);
         t+=length[i+1];
      }
   *(high->data+high->length)='\0';

}
