/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright (c) 1999 Ming Lu, computer science, cornell university
 * All rights reserved.
 *
 * This package can be used for non-profit, non-commercial
 * purpose. For any other purpose, please contact the author at 
 * ml124@cornell.edu
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

extern "C"{

#include "io-util.h"
ppm_gets(FILE *fdin,unsigned char (*s)[])
{
   int i,done;
   unsigned char *t;
   done=0;
   while(!done){
      fscanf(fdin,"%s", s);
      for(i=0; !done; i++){
	 if((*s)[i] == '#') {
	    fgets(*s,256,fdin);
	    break;
	 }
	 if(!isspace((*s)[i])) done=1;
      }
   }
}

   

get_pgm_image_size(unsigned char *name, int *inrows, int *incols)
{
   int range;
   unsigned char  imgtype[4], ins[256];
   FILE *fdin;

   if ((fdin = fopen(name,"r")) == NULL)
       {
		fprintf(stderr,"cannot open file %s\n enter new name->", 
			name);
		gets(name);
		if ((fdin = fopen(name,"r")) == NULL)
		    {
		       
		       fprintf(stderr,
			       "cannot open file %s either, aborting\n", 
			       name);		
		       exit(-1);
		    }
	     }

   fscanf(fdin,"%s\n",imgtype);
   ppm_gets(fdin,&ins);
   sscanf(ins,"%d", incols);

   ppm_gets(fdin,&ins);
   sscanf(ins,"%d", inrows);

   ppm_gets(fdin,&ins);
   sscanf(ins,"%d", &range);

  /* get rows, cols and range */

   if(*inrows <= 0 || *incols <= 0)
	 fprintf(stderr,": negative rows or columns. Check!! %s",name);
   fclose(fdin);

}



read_pgm_BYTE(unsigned char *name,unsigned char **image, int *rows, int *cols)

{
   int i,j, range;
   unsigned char *tpt, imgtype[4], ins[257] ;
   FILE *fdin;

   if ((fdin = fopen(name,"r")) == NULL)
       {
		fprintf(stderr,"cannot open file %s\n enter new name->", 
			name);
		gets(name);
		if ((fdin = fopen(name,"r")) == NULL)
		    {
		       
		       fprintf(stderr,
			       "cannot open file %s either, aborting\n", 
			       name);		
		       exit(-1);
		    }
	     }


   fscanf(fdin,"%s\n",imgtype);
   ppm_gets(fdin,&ins);
   sscanf(ins,"%d", cols);

   ppm_gets(fdin,&ins);
   sscanf(ins,"%d", rows);

   ppm_gets(fdin,&ins);
   sscanf(ins,"%d", &range);

   if(*rows <= 0 || *cols <= 0)
	 fprintf(stderr,": negative rows or columns. Check!! %s",name);
   *image = (unsigned char *)
	 MALLOC((unsigned) ((*rows+1) * (*cols+1) * sizeof(unsigned char)));
   if (*image == NULL) {
      fprintf(stderr, "can't allocate memory for image %s\n",name);
      exit(-1);
   }
	 

   
   if(strcmp(imgtype,"P5") == 0 || strcmp(imgtype,"p5") == 0)
       { /* read binary form */
	  tpt = *image;
	  for (i = 0; i < *rows; i++)
	      {
		 for (j = 0; j < *cols; j++,tpt++)
		     {
			if(EOF == fscanf(fdin,"%c",  tpt)) 
			      fprintf(stderr,
				      "WARNING!! not enought bytes for %s, on r%d c%d\n",
				      name, i,j);
		     }
	      }
       }  else   if(strcmp(imgtype,"P2") == 0  || strcmp(imgtype,"p2") == 0) 
	   {     /* its ascii form */
	      tpt = *image;
	      for (i = 0; i < *rows; i++)
		  {
		     for (j = 0; j <= *cols; j++)
			 {
			    if(EOF == fscanf(fdin,"%d", tpt)) 
			      fprintf(stderr,
				      "WARNING!! not enought bytes for %s, on r%d c%d\n",
				      name, i,j);
			    tpt++;
		     }
	      }
	   } else    { /* not p2 or p5 */
	      fprintf(stderr,"ERROR, called read pgm_byte but %s not a P2 or P5 image, boirting\n",
		      name);
		  
	   }
   fclose(fdin);

}


write_pgm_BYTE(unsigned char*name,unsigned char *image, int rows, int cols, unsigned char *comment)

{
   int i,j;
   unsigned char *tpt, fname[256];
   FILE *fdout;

  if(strlen(name) <1)
      {
	 fprintf(stderr,"Image write called withoutd filename\n");
	 fprintf(stderr,"Please input filename->");
	 scanf("%s",fname);
    } else strcpy(fname, name);


   if ((fdout = fopen(fname,"w")) == NULL)
       {
	  fprintf(stderr,"cannot open file >>%s<< for output\n", fname);
	  fprintf(stderr,"continuing without write\n");
	  return;
  }


   fprintf(fdout,"P5\n#%s\n%d %d\n255\n",comment, cols,rows);
   tpt = image;
   for (i = 0; i < rows; i++)
       {
	  for (j = 0; j <= cols; j++)
	      {
		 fprintf(fdout,"%c", (unsigned char) *tpt++);
	      }
       }
   printf("\n");
   fclose(fdout);
}








void write_ppm_BYTE(unsigned char *filename,unsigned char *data,int rows, int cols,   unsigned char *comment)
{
   FILE *fp;
   fp=fopen((char *)filename,"w");
   if(fp==NULL)
      {
         fprintf(fp,"FILE IO Error : %s \n",filename);
      }
   fprintf(fp,"P6\n#%s\n%d %d\n255\n",(char *)comment,cols,rows);
   unsigned char *t;
   t=data;
   
   for(int i=0;i<cols*rows*3;i++)
      fprintf(fp,"%c",data[i]);

   fclose(fp);
}



int read_ppm_BYTE(unsigned char *name,unsigned char **image, int *rows, int *cols)

{
   int i,j, range;
   unsigned char *tpt, imgtype[4], ins[257] ;
   FILE *fdin;

   if ((fdin = fopen(name,"r")) == NULL)
       {
		fprintf(stderr,"cannot open file %s\n enter new name->", 
			name);
		gets(name);
		if ((fdin = fopen(name,"r")) == NULL)
		    {
		       
		       fprintf(stderr,
			       "cannot open file %s either, aborting\n", 
			       name);		
		       exit(-1);
		    }
	     }


   fscanf(fdin,"%s\n",imgtype);
   ppm_gets(fdin,&ins);
   sscanf(ins,"%d", cols);

   ppm_gets(fdin,&ins);
   sscanf(ins,"%d", rows);

   ppm_gets(fdin,&ins);
   sscanf(ins,"%d", &range);

   if(*rows <= 0 || *cols <= 0)
	 fprintf(stderr,": negative rows or columns. Check!! %s",name);
   *image = (unsigned char *)
	 MALLOC((unsigned) ((*rows+1) * (*cols+1) * sizeof(unsigned char))*3);
   if (*image == NULL) {
      fprintf(stderr, "can't allocate memory for image %s\n",name);
      exit(-1);
   }
	 

   
   if(strcmp(imgtype,"P6") == 0 || strcmp(imgtype,"p6") == 0)
       { /* read binary form */
	  tpt = *image;
	  for (i = 0; i < *rows; i++)
	      {
		 for (j = 0; j < *cols; j++,tpt+=3)
		     {
			if(EOF == fscanf(fdin,"%c%c%c",  tpt,tpt+1,tpt+2)) 
			      fprintf(stderr,
				      "WARNING!! not enought bytes for %s, on r%d c%d\n",
				      name, i,j);
		     }
	      }
       }  else   if(strcmp(imgtype,"P3") == 0  || strcmp(imgtype,"p3") == 0) 
	   {     /* its ascii form */
	      tpt = *image;
	      for (i = 0; i < *rows; i++)
		  {
		     for (j = 0; j <= *cols; j++)
			 {
			    if(EOF == fscanf(fdin,"%d%d%d", tpt,tpt+1,tpt+2)) 
			      fprintf(stderr,
				      "WARNING!! not enought bytes for %s, on r%d c%d\n",
				      name, i,j);
			    tpt+=3;
		     }
	      }
	   } else    { /* not p3 or p6 */
	      fprintf(stderr,"ERROR, called read pgm_byte but %s not a P3 or P6 image, boirting\n",
		      name);
		  
	   }
   fclose(fdin);

}
int  read_ppm_BYTE_withpreallcatedMemory(unsigned char *name,unsigned char *image, int *rows, int *cols)

{
   int i,j, range;
   unsigned char *tpt, imgtype[4], ins[257] ;
   FILE *fdin;

   if ((fdin = fopen(name,"r")) == NULL)
       {
		fprintf(stderr,"cannot open file %s\n enter new name->", 
			name);
		gets(name);
		if ((fdin = fopen(name,"r")) == NULL)
		    {
		       
		       fprintf(stderr,
			       "cannot open file %s either, aborting\n", 
			       name);		
		       exit(-1);
		    }
	     }


   fscanf(fdin,"%s\n",imgtype);
   ppm_gets(fdin,&ins);
   sscanf(ins,"%d", cols);

   ppm_gets(fdin,&ins);
   sscanf(ins,"%d", rows);

   ppm_gets(fdin,&ins);
   sscanf(ins,"%d", &range);
   
   if(strcmp(imgtype,"P6") == 0 || strcmp(imgtype,"p6") == 0)
       { /* read binary form */
	  tpt = image;
	  for (i = 0; i < *rows; i++)
	      {
		 for (j = 0; j < *cols; j++,tpt+=3)
		     {
			if(EOF == fscanf(fdin,"%c%c%c",  tpt,tpt+1,tpt+2)) 
			      fprintf(stderr,
				      "WARNING!! not enought bytes for %s, on r%d c%d\n",
				      name, i,j);
		     }
	      }
       }  else   if(strcmp(imgtype,"P3") == 0  || strcmp(imgtype,"p3") == 0) 
	   {     /* its ascii form */
	      tpt = image;
	      for (i = 0; i < *rows; i++)
		  {
		     for (j = 0; j <= *cols; j++)
			 {
			    if(EOF == fscanf(fdin,"%d%d%d", tpt,tpt+1,tpt+2)) 
			      fprintf(stderr,
				      "WARNING!! not enought bytes for %s, on r%d c%d\n",
				      name, i,j);
			    tpt+=3;
		     }
	      }
	   } else    { /* not p3 or p6 */
	      fprintf(stderr,"ERROR, called read pgm_byte but %s not a P3 or P6 image, boirting\n",
		      name);
		  
	   }
   fclose(fdin);

}
 
};

